package eta_business

import (
	"github.com/beego/beego/v2/client/orm"
	"time"
)

type BusinessChartClassifyPermission struct {
	ChartClassifyPermissionId int       `orm:"column(chart_classify_permission_id);pk"`
	ChartClassifyId           int       `description:"分类Id"`
	Source                    int       `description:"分类来源: 1-图库; 2-商品价格曲线; 3-相关性图表; 6-拟合方程图表; 7-统计特征"`
	BusinessCode              string    `description:"商户号"`
	AdminId                   int       `description:"系统用户Id"`
	ModifyTime                time.Time `description:"变更时间"`
	CreateTime                time.Time `description:"关系建立时间"`
}

func AddBusinessChartClassifyPermission(list []*BusinessChartClassifyPermission, businessCode string) (err error) {
	// 事务，先执行删除，再新增
	o := orm.NewOrm()

	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	sql := ` DELETE FROM business_chart_classify_permission WHERE business_code = ?`
	_, err = to.Raw(sql, businessCode).Exec()
	if err != nil {
		return
	}
	if len(list) > 0 {
		_, err = o.InsertMulti(len(list), list)
		return
	}
	return
}

type AddBusinessChartClassifyPermissionReq struct {
	EtaBusinessId      int    `description:"商家ID"`
	ChartClassifyIdStr string `description:"图表分类ID,多个用英文逗号隔开"`
}
