package mgodb

import (
	"context"
	"eta/eta_forum_admin/utils"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo/options"
	"time"
)

type EdbDataBase struct {
	EdbDataId     primitive.ObjectID `bson:"_id,omitempty"`
	EdbInfoId     int                `bson:"edb_info_id"`
	EdbCode       string             `bson:"edb_code"`
	DataTime      time.Time          `bson:"data_time"`
	Value         float64            `bson:"value"`
	Status        int                `bson:"status"`
	CreateTime    time.Time          `bson:"create_time"`
	ModifyTime    time.Time          `bson:"modify_time"`
	DataTimestamp int64              `bson:"data_timestamp"`
}

// GetEdbDataList 获取指标的数据（日期正序返回）
func GetEdbDataList(endInfoId int, startDate, endDate time.Time) (list []*EdbDataBase, err error) {
	findOptions := options.Find()
	findOptions.SetSort(bson.D{{"data_time", 1}})
	db := NewMgo(utils.MgoDataDbName, "edb_data_base", utils.MgoDataCli)
	filter := bson.D{{"edb_info_id", endInfoId}}
	if !startDate.IsZero() {
		filter = append(filter, bson.E{"data_time", bson.M{"$gte": startDate}})
	}
	if !endDate.IsZero() {
		filter = append(filter, bson.E{"data_time", bson.M{"$lte": endDate}})
	}
	ctx := context.TODO()
	cur, err := db.Find(filter, findOptions)
	if err != nil {
		return
	}
	// Close the cursor once finished
	defer cur.Close(ctx)
	for cur.Next(ctx) {
		// create a value into which the single document can be decoded
		var elem EdbDataBase
		err = cur.Decode(&elem)
		if err != nil {
			return
		}
		elem.DataTime = elem.DataTime.In(time.Local)
		elem.CreateTime = elem.CreateTime.In(time.Local)
		elem.ModifyTime = elem.ModifyTime.In(time.Local)
		list = append(list, &elem)
	}

	if err = cur.Err(); err != nil {
		return
	}
	return
}
