package utils

import (
	"io/fs"
	"time"
)

// 常量定义
const (
	FormatTime                 = "15:04:05"                //时间格式
	FormatDate                 = "2006-01-02"              //日期格式
	FormatDateUnSpace          = "20060102"                //日期格式
	FormatDateTime             = "2006-01-02 15:04:05"     //完整时间格式
	HlbFormatDateTime          = "2006-01-02_15:04:05.999" //完整时间格式
	FormatDateTimeUnSpace      = "20060102150405"          //完整时间格式
	FormatShortDateTimeUnSpace = "060102150405"            //省去开头两位年份的时间格式
	FormatYearMonthDate        = "2006-01"                 //日期格式
	FormatYearMonthUnSpace     = "200601"                  //年月的日期格式
	PageSize15                 = 15                        //列表页每页数据量
	PageSize5                  = 5
	PageSize10                 = 10
	PageSize20                 = 20
	PageSize30                 = 30
)
const DateTypeNYears = 20 //时间类型为最近N年
// 数据来源渠道
const (
	DATA_SOURCE_THS                                  = iota + 1 //同花顺
	DATA_SOURCE_WIND                                            //wind->2
	DATA_SOURCE_PB                                              //彭博->3
	DATA_SOURCE_CALCULATE                                       //指标运算->4
	DATA_SOURCE_CALCULATE_LJZZY                                 //累计值转月->5
	DATA_SOURCE_CALCULATE_TBZ                                   //同比值->6
	DATA_SOURCE_CALCULATE_TCZ                                   //同差值->7
	DATA_SOURCE_CALCULATE_NSZYDPJJS                             //N数值移动平均计算->8
	DATA_SOURCE_MANUAL                                          //手工指标->9
	DATA_SOURCE_LZ                                              //隆众->10
	DATA_SOURCE_YS                                              //有色->11
	DATA_SOURCE_CALCULATE_HBZ                                   //环比值->12
	DATA_SOURCE_CALCULATE_HCZ                                   //环差值->13
	DATA_SOURCE_CALCULATE_BP                                    //变频->14
	DATA_SOURCE_GL                                              //钢联->15
	DATA_SOURCE_ZZ                                              //郑商所->16
	DATA_SOURCE_DL                                              //大商所->17
	DATA_SOURCE_SH                                              //上期所->18
	DATA_SOURCE_CFFEX                                           //中金所->19
	DATA_SOURCE_SHFE                                            //上期能源->20
	DATA_SOURCE_GIE                                             //欧洲天然气->21
	DATA_SOURCE_CALCULATE_TIME_SHIFT                            //时间移位->22
	DATA_SOURCE_CALCULATE_ZJPJ                                  //直接拼接->23
	DATA_SOURCE_CALCULATE_LJZTBPJ                               //累计值同比拼接->24
	DATA_SOURCE_LT                                              //路透->25
	DATA_SOURCE_COAL                                            //中国煤炭市场网->26
	DATA_SOURCE_PYTHON                                          //python代码->27
	DATA_SOURCE_PB_FINANCE                                      //彭博财务数据->28
	DATA_SOURCE_GOOGLE_TRAVEL                                   //谷歌出行->29
	DATA_SOURCE_PREDICT                                         //普通预测指标->30
	DATA_SOURCE_PREDICT_CALCULATE                               //预测指标运算->31
	DATA_SOURCE_PREDICT_CALCULATE_TBZ                           //预测同比值->32
	DATA_SOURCE_PREDICT_CALCULATE_TCZ                           //预测同差值->33
	DATA_SOURCE_MYSTEEL_CHEMICAL                                //钢联化工->34
	DATA_SOURCE_CALCULATE_CJJX                                  //超季节性->35
	DATA_SOURCE_EIA_STEO                                        //eia steo报告->36
	DATA_SOURCE_CALCULATE_NHCC                                  //计算指标（拟合残差）->37
	DATA_SOURCE_COM_TRADE                                       //联合国商品贸易数据->38
	DATA_SOURCE_PREDICT_CALCULATE_NSZYDPJJS                     //预测指标 - N数值移动平均计算 -> 39
	DATA_SOURCE_CALCULATE_ADJUST                                //数据调整->40
	DATA_SOURCE_SCI                                             //卓创数据（红桃三）->41
	DATA_SOURCE_PREDICT_CALCULATE_LJZZY                         //预测指标 - 累计值转月->42
	DATA_SOURCE_PREDICT_CALCULATE_HBZ                           //预测指标 - 环比值->43
	DATA_SOURCE_PREDICT_CALCULATE_HCZ                           //预测指标 - 环差值->44
	DATA_SOURCE_PREDICT_CALCULATE_BP                            //预测指标 - 变频->45
	DATA_SOURCE_PREDICT_CALCULATE_TIME_SHIFT                    //预测指标 - 时间移位->46
	DATA_SOURCE_PREDICT_CALCULATE_ZJPJ                          //预测指标 - 直接拼接->47
	DATA_SOURCE_PREDICT_CALCULATE_LJZTBPJ                       //预测指标 - 累计值同比拼接->48
	DATA_SOURCE_PREDICT_CALCULATE_CJJX                          //预测指标 - 超季节性->49
	DATA_SOURCE_PREDICT_CALCULATE_NHCC                          //预测指标 - 计算指标（拟合残差）->50
	DATA_SOURCE_CALCULATE_JP                                    //变频->51
	DATA_SOURCE_CALCULATE_NH                                    //年化->52
	DATA_SOURCE_CALCULATE_KSZS                                  //扩散指数->53
	DATA_SOURCE_PREDICT_CALCULATE_JP                            //预测指标 - 计算指标(降频)->54
	DATA_SOURCE_PREDICT_CALCULATE_NH                            //预测指标 - 计算指标(年化)->55
	DATA_SOURCE_PREDICT_CALCULATE_KSZS                          //预测指标 - 计算指标(扩散指数)->56
	DATA_SOURCE_BAIINFO                                         //百川盈孚 ->57
	DATA_SOURCE_STOCK_PLANT                                     //存量装置 ->58
	DATA_SOURCE_CALCULATE_CORRELATION                           //滚动相关性->59
	DATA_SOURCE_NATIONAL_STATISTICS                             //国家统计局->60
	DATA_SOURCE_CALCULATE_LJZZJ                                 //累计值转季 -> 61
	DATA_SOURCE_CALCULATE_LJZ                                   //累计值 -> 62
	DATA_SOURCE_CALCULATE_LJZNCZJ                               //累计值(年初至今） -> 63
	DATA_SOURCE_PREDICT_CALCULATE_LJZZJ                         //预测指标 - 累计值转季->64
	DATA_SOURCE_PREDICT_CALCULATE_LJZ                           //预测指标 - 累计值 -> 65
	DATA_SOURCE_PREDICT_CALCULATE_LJZNCZJ                       //预测指标 - 累计值(年初至今） -> 66
	DATA_SOURCE_CALCULATE_STANDARD_DEVIATION                    //标准差->67
	DATA_SOURCE_CALCULATE_PERCENTILE                            //百分位图表->68
	DATA_SOURCE_PREDICT_CALCULATE_STANDARD_DEVIATION            //预测标准差->69
	DATA_SOURCE_PREDICT_CALCULATE_PERCENTILE                    //预测百分位->70
	DATA_SOURCE_FUBAO                                           //富宝->71
	DATA_SOURCE_CALCULATE_ZSXY                                  // 指数修匀->72
	DATA_SOURCE_PREDICT_CALCULATE_ZSXY                          // 预测指数修匀->73
	DATA_SOURCE_CALCULATE_ZDYFX                                 // 自定义分析->74
	DATA_SOURCE_CALCULATE_RJZ                                   // 日均值计算->75
	DATA_SOURCE_GFEX                                 = 78       // 广州期货交易所->78
	DATA_SOURCE_ICPI                                 = 79       // ICPI消费价格指数->79
)

// 子数据来源渠道
const (
	DATA_SUB_SOURCE_EDB  = iota //经济数据库
	DATA_SUB_SOURCE_DATE        //日期序列
)

// 指标来源的中文展示
const (
	DATA_SOURCE_NAME_THS                                  = `同花顺`               //同花顺
	DATA_SOURCE_NAME_WIND                                 = `wind`              //wind
	DATA_SOURCE_NAME_PB                                   = `彭博`                //彭博
	DATA_SOURCE_NAME_CALCULATE                            = `指标运算`              //指标运算
	DATA_SOURCE_NAME_CALCULATE_LJZZY                      = `累计值转月值`            //累计值转月
	DATA_SOURCE_NAME_CALCULATE_TBZ                        = `同比值`               //同比值
	DATA_SOURCE_NAME_CALCULATE_TCZ                        = `同差值`               //同差值
	DATA_SOURCE_NAME_CALCULATE_NSZYDPJJS                  = `N数值移动平均计算`         //N数值移动平均计算
	DATA_SOURCE_NAME_MANUAL                               = `手工数据`              //手工指标
	DATA_SOURCE_NAME_LZ                                   = `隆众`                //隆众
	DATA_SOURCE_NAME_YS                                   = `SMM`               //有色
	DATA_SOURCE_NAME_CALCULATE_HBZ                        = `环比值`               //环比值->12
	DATA_SOURCE_NAME_CALCULATE_HCZ                        = `环差值`               //环差值->13
	DATA_SOURCE_NAME_CALCULATE_BP                         = `升频`                //变频,2023-2-10 13:56:01调整为"升频"->14
	DATA_SOURCE_NAME_GL                                   = `钢联`                //钢联->15
	DATA_SOURCE_NAME_ZZ                                   = `郑商所`               //郑商所->16
	DATA_SOURCE_NAME_DL                                   = `大商所`               //大商所->17
	DATA_SOURCE_NAME_SH                                   = `上期所`               //上期所->18
	DATA_SOURCE_NAME_CFFEX                                = `中金所`               //中金所->19
	DATA_SOURCE_NAME_SHFE                                 = `上期能源`              //上期能源->20
	DATA_SOURCE_NAME_GIE                                  = `欧洲天然气`             //欧洲天然气->21
	DATA_SOURCE_NAME_CALCULATE_TIME_SHIFT                 = `时间移位`              //时间移位->22
	DATA_SOURCE_NAME_CALCULATE_ZJPJ                       = `直接拼接`              //直接拼接->23
	DATA_SOURCE_NAME_CALCULATE_LJZTBPJ                    = `累计值同比拼接`           //累计值同比拼接->24
	DATA_SOURCE_NAME_LT                                   = `路透`                //路透->25
	DATA_SOURCE_NAME_COAL                                 = `中国煤炭网`             //煤炭网->26
	DATA_SOURCE_NAME_PYTHON                               = `代码运算`              //python代码->27
	DATA_SOURCE_NAME_PB_FINANCE                           = `彭博财务`              //彭博财务数据->28
	DATA_SOURCE_NAME_GOOGLE_TRAVEL                        = `our world in data` //谷歌出行数据->29
	DATA_SOURCE_NAME_PREDICT                              = `预测指标`              //普通预测指标->30
	DATA_SOURCE_NAME_PREDICT_CALCULATE                    = `预测指标运算`            //预测指标运算->31
	DATA_SOURCE_NAME_PREDICT_CALCULATE_TBZ                = `预测同比`              //预测指标 - 同比值->32
	DATA_SOURCE_NAME_PREDICT_CALCULATE_TCZ                = `预测同差`              //预测指标 - 同差值->33
	DATA_SOURCE_NAME_MYSTEEL_CHEMICAL                     = `钢联化工`              //钢联化工->34
	DATA_SOURCE_NAME_CALCULATE_CJJX                       = `超季节性`              //超季节性->35
	DATA_SOURCE_NAME_EIA_STEO                             = `EIA STERO报告`       //eia stero报告->36
	DATA_SOURCE_NAME_CALCULATE_NHCC                       = `拟合残差`              //计算指标（拟合残差）->37
	DATA_SOURCE_NAME_COM_TRADE                            = `UN`                //联合国商品贸易数据->38
	DATA_SOURCE_NAME_PREDICT_CALCULATE_NSZYDPJJS          = `预测N数值移动平均计算`       //预测指标 - N数值移动平均计算 -> 39
	DATA_SOURCE_NAME_CALCULATE_ADJUST                     = `数据调整`              //数据调整->40
	DATA_SOURCE_NAME_SCI                                  = `SCI`               //卓创数据（红桃三）->41
	DATA_SOURCE_NAME_PREDICT_CALCULATE_LJZZY              = `预测累计值转月值`          //预测指标 - 累计值转月->42
	DATA_SOURCE_NAME_PREDICT_CALCULATE_HBZ                = `预测环比值`             //预测指标 - 环比值->43
	DATA_SOURCE_NAME_PREDICT_CALCULATE_HCZ                = `预测环差值`             //预测指标 - 环差值->44
	DATA_SOURCE_NAME_PREDICT_CALCULATE_BP                 = `预测升频`              //预测指标 - 升频->45
	DATA_SOURCE_NAME_PREDICT_CALCULATE_TIME_SHIFT         = `预测时间移位`            //预测指标 - 时间移位->46
	DATA_SOURCE_NAME_PREDICT_CALCULATE_ZJPJ               = `预测直接拼接`            //预测指标 - 直接拼接->47
	DATA_SOURCE_NAME_PREDICT_CALCULATE_LJZTBPJ            = `预测累计值同比拼接`         //预测指标 - 累计值同比拼接->48
	DATA_SOURCE_NAME_PREDICT_CALCULATE_CJJX               = `预测超季节性`            //预测指标 - 超季节性->49
	DATA_SOURCE_NAME_PREDICT_CALCULATE_NHCC               = `预测拟合残差`            //预测指标 - 计算指标（拟合残差）->50
	DATA_SOURCE_NAME_CALCULATE_JP                         = `降频`                //降频->51
	DATA_SOURCE_NAME_CALCULATE_NH                         = `年化`                //年化->52
	DATA_SOURCE_NAME_CALCULATE_KSZS                       = `扩散指数`              //扩散指数->53
	DATA_SOURCE_NAME_PREDICT_CALCULATE_JP                 = `预测降频`              //预测指标 - 计算指标(降频)->54
	DATA_SOURCE_NAME_PREDICT_CALCULATE_NH                 = `预测年化`              //预测指标 - 计算指标(年化)->55
	DATA_SOURCE_NAME_PREDICT_CALCULATE_KSZS               = `预测扩散指数`            //预测指标 - 计算指标(扩散指数)->56
	DATA_SOURCE_NAME_BAIINFO                              = `百川盈孚`              //百川盈孚 ->57
	DATA_SOURCE_NAME_STOCK_PLANT                          = `存量装置`              //存量装置 ->58
	DATA_SOURCE_NAME_CALCULATE_CORRELATION                = `相关性计算`             //相关性计算->59
	DATA_SOURCE_NAME_NATIONAL_STATISTICS                  = `国家统计局`             //国家统计局->60
	DATA_SOURCE_NAME_CALCULATE_LJZZJ                      = `累计值转季值`            //累计值转季 -> 61
	DATA_SOURCE_NAME_CALCULATE_LJZ                        = `累计值`               //累计值 -> 62
	DATA_SOURCE_NAME_CALCULATE_LJZNCZJ                    = `年初至今累计值`           //累计值(年初至今） -> 63
	DATA_SOURCE_NAME_PREDICT_CALCULATE_LJZZJ              = `预测累计值转季值`          //预测指标 - 累计值转季->64
	DATA_SOURCE_NAME_PREDICT_CALCULATE_LJZ                = `预测累计值`             //预测指标 - 累计值 -> 65
	DATA_SOURCE_NAME_PREDICT_CALCULATE_LJZNCZJ            = `预测年初至今累计值`         //预测指标 - 累计值(年初至今） -> 66
	DATA_SOURCE_NAME_CALCULATE_STANDARD_DEVIATION         = `标准差`               //标准差->67
	DATA_SOURCE_NAME_CALCULATE_PERCENTILE                 = `百分位`               //百分位->68
	DATA_SOURCE_NAME_PREDICT_CALCULATE_STANDARD_DEVIATION = `预测标准差`             //预测标准差->69
	DATA_SOURCE_NAME_PREDICT_CALCULATE_PERCENTILE         = `预测百分位`             //预测百分位->70
	DATA_SOURCE_NAME_FUBAO                                = `富宝数据`              //富宝数据->71
)

// 基础数据初始化日期
var (
	BASE_START_DATE         = time.Now().AddDate(-30, 0, 0).Format(FormatDate)        //基础数据开始日期
	BASE_END_DATE           = time.Now().AddDate(4, 0, 0).Format(FormatDate)          //基础数据结束日期
	BASE_START_DATE_UnSpace = time.Now().AddDate(-30, 0, 0).Format(FormatDateUnSpace) //基础数据开始日期
	BASE_END_DATE_UnSpace   = time.Now().AddDate(4, 0, 0).Format(FormatDateUnSpace)   //基础数据结束日期
)

const (
	DATA_PREFIX  = "hz_data"
	CHART_PREFIX = "hz_chart"
)

// 数据刷新频率
const (
	DATA_REFRESH        = 7 //7个单位，日/周/月/季度/年
	DATA_END_DATE_LIMIT = 4 //数据结束日期为，当前日期，加上4年时间
)

const (
	CACHE_EDB_DATA_ADD          = "CACHE_EDB_DATA_ADD_"
	CACHE_EDB_DATA_REFRESH      = "CACHE_EDB_DATA_REFRESH_"
	CACHE_WIND_URL              = "CACHE_WIND_URL"
	CACHE_CHART_INFO_DATA       = "chart:info:data:"             //图表数据
	CACHE_STOCK_PLANT_CALCULATE = "CACHE_STOCK_PLANT_CALCULATE_" // 库存装置减产计算
)

// MonthQuarterMap 月份与季度的map
var MonthQuarterMap = map[int]int{
	1:  1,
	2:  1,
	3:  1,
	4:  2,
	5:  2,
	6:  2,
	7:  3,
	8:  3,
	9:  3,
	10: 4,
	11: 4,
	12: 4,
}

// FrequencyDaysMap 频度日期的map关系
var FrequencyDaysMap = map[string]int{
	"天": 1, "周": 7, "月": 30, "季": 90, "年": 365,
}

// 角色类型/类型编码
const (
	ROLE_TYPE_ADMIN       = "管理员"
	ROLE_TYPE_FICC_ADMIN  = "ficc管理员"
	ROLE_TYPE_FICC_SELLER = "ficc销售"
	ROLE_TYPE_RAI_ADMIN   = "权益管理员"
	ROLE_TYPE_RAI_SELLER  = "权益销售"

	ROLE_TYPE_FICC_GROUP      = "ficc销售组长"
	ROLE_TYPE_FICC_MANAGER    = "ficc销售主管"
	ROLE_TYPE_RAI_GROUP       = "权益组长"
	ROLE_TYPE_FICC_DEPARTMENT = "ficc部门经理"
	ROLE_TYPE_RAI_DEPARTMENT  = "权益部门经理"
	ROLE_TYPE_FICC_RESEARCHR  = "ficc研究员"
	ROLE_TYPE_RAI_RESEARCHR   = "权益研究员"
	ROLE_NAME_FICC_DIRECTOR   = "ficc销售经理" // 实际角色类型为ficc销售主管

	ROLE_TYPE_CODE_ADMIN           = "admin"           //管理员
	ROLE_TYPE_CODE_FICC_ADMIN      = "ficc_admin"      //ficc管理员
	ROLE_TYPE_CODE_FICC_SELLER     = "ficc_seller"     //ficc销售
	ROLE_TYPE_CODE_RAI_ADMIN       = "rai_admin"       //权益管理员
	ROLE_TYPE_CODE_RAI_SELLER      = "rai_seller"      //权益销售
	ROLE_TYPE_CODE_FICC_GROUP      = "ficc_group"      //ficc销售主管
	ROLE_TYPE_CODE_RAI_GROUP       = "rai_group"       //ficc组长
	ROLE_TYPE_CODE_FICC_DEPARTMENT = "ficc_department" //ficc部门经理
	ROLE_TYPE_CODE_RAI_DEPARTMENT  = "rai_department"  //权益部门经理
	ROLE_TYPE_CODE_FICC_RESEARCHR  = "ficc_researcher" //ficc研究员
	ROLE_TYPE_CODE_RESEARCHR       = "researcher"      //ficc研究员（最早定义的）
	ROLE_TYPE_CODE_RAI_RESEARCHR   = "rai_researcher"  //权益研究员
	ROLE_TYPE_CODE_COMPLIANCE      = "compliance"      //合规角色
	ROLE_TYPE_CODE_FINANCE         = "finance"         //财务角色
	ROLE_TYPE_CODE_FICC_TEAM       = "ficc_team"       //ficc销售组长
)
const (
	COMPANY_PRODUCT_FICC_ID   = 1
	COMPANY_PRODUCT_FICC_NAME = "ficc"
	COMPANY_PRODUCT_RAI_ID    = 2
	COMPANY_PRODUCT_RAI_NAME  = "权益"
)

// 语言版本
const (
	ZhLangVersion = "zh" // 中文语言版本
	EnLangVersion = "en" // 英文语言版本
)

// redis缓存
const (
	CACHE_ACCESS_TOKEN_LOGIN          = "eta_forum_admin:login:"                    //管理后台登录
	CACHE_ACCESS_TOKEN_LOGIN_NO_TRUST = "eta_forum_admin:login:no_trust:"           //管理后台登录（不可信登录态）
	CACHE_CHART_CLASSIFY              = "eta_forum_admin:chart:classify"            //图表分类数据
	LoginCacheTime                    = 600                                         // 登录缓存时长, 分钟
	CACHE_KEY_ADMIN_OPERATE_RECORD    = "eta_forum_admin:list:admin_operate_record" //系统用户操作日志队列
)

const (
	UserLoginSalt = "MiQM9yusNA9T2uIH"         // 用户登录盐值
	DesKeySalt    = "JMCqSoUrTAmyNNIRb0TtlrPk" // DesKey盐值
)

const (
	CACHE_CRM_AUTH_CODE_PREFIX   = "eta_forum:crm_auth_code:"   // 免密登录Code-CRM
	CACHE_FORUM_AUTH_CODE_PREFIX = "eta_forum:forum_auth_code:" // 免密登录Code-ETA
)

// 图表样式类型
const (
	CHART_TYPE_CURVE           = 1  //曲线图
	CHART_TYPE_SEASON          = 2  //季节性图
	CHART_TYPE_BAR             = 7  //柱形图
	CHART_TYPE_SECTION_SCATTER = 10 //截面散点图样式
	CHART_TYPE_RADAR           = 11 //雷达图
	CHART_TYPE_SECTION_COMBINE = 14 //截面组合图
)

// 图表类型
const (
	CHART_SOURCE_DEFAULT                         = 1
	CHART_SOURCE_FUTURE_GOOD                     = 2
	CHART_SOURCE_CORRELATION                     = 3  // 相关性图表
	CHART_SOURCE_ROLLING_CORRELATION             = 4  // 滚动相关性图表
	CHART_SOURCE_FUTURE_GOOD_PROFIT              = 5  // 商品利润曲线
	CHART_SOURCE_LINE_EQUATION                   = 6  // 拟合方程图表
	CHART_SOURCE_LINE_FEATURE_STANDARD_DEVIATION = 7  // 统计特征-标准差图表
	CHART_SOURCE_LINE_FEATURE_PERCENTILE         = 8  // 统计特征-百分位图表
	CHART_SOURCE_LINE_FEATURE_FREQUENCY          = 9  // 统计特征-频率分布图表
	CHART_SOURCE_CROSS_HEDGING                   = 10 // 跨品种分析图表
)

// DataSourceEnMap 指标来源的英文名称
var DataSourceEnMap = map[int]string{
	DATA_SOURCE_WIND:             "Wind",
	DATA_SOURCE_THS:              "iFind",
	DATA_SOURCE_PB:               "Bloomberg",
	DATA_SOURCE_PB_FINANCE:       "Bloomberg Finance",
	DATA_SOURCE_LT:               "Reuters",
	DATA_SOURCE_MANUAL:           "Horizon Insights",
	DATA_SOURCE_LZ:               "OilChem",
	DATA_SOURCE_YS:               "SMM",
	DATA_SOURCE_GL:               "MySteel",
	DATA_SOURCE_ZZ:               "Zhengzhou Commodity Exchange",
	DATA_SOURCE_DL:               "Dalian Commodity Exchange",
	DATA_SOURCE_SH:               "Shanghai Futures Exchange",
	DATA_SOURCE_CFFEX:            "China Financial Futures Exchange",
	DATA_SOURCE_SHFE:             "Shanghai International Energy Exchange",
	DATA_SOURCE_GIE:              "Eurostat",
	DATA_SOURCE_COAL:             "China Coal Transport & Distribution Association",
	DATA_SOURCE_GOOGLE_TRAVEL:    "Our World in Data",
	DATA_SOURCE_EIA_STEO:         "Energy Information Administration",
	DATA_SOURCE_COM_TRADE:        "United Nations",
	DATA_SOURCE_SCI:              "Sublime China Information",
	DATA_SOURCE_BAIINFO:          "BAIINFO",
	DATA_SOURCE_MYSTEEL_CHEMICAL: "Horizon Insights",
	DATA_SOURCE_FUBAO:            "FuBao",
}

// BusinessCodeSalt 商家编码盐值
const BusinessCodeSalt = "dr7WY0OZgGR7upw1"

//const CrmEtaAuthorization = "NIi1RbEmH0C2rksXtPGDPBBgRgTZY87Q"

const HzCompanyName = "弘则研究"
const HzBusinessCodeRelease = "E2023080900" // 生产环境

// DIR_MOD 目录创建权限
const DIR_MOD fs.FileMode = 0766 // Unix permission bits
const (
	STORAGESOURCE_OSS_NAME   = "oss"
	STORAGESOURCE_MINIO_NAME = "minio"
	STORAGESOURCE_S3_NAME    = "s3"
)

// 客户类型
const (
	COMPANY_CLASSIFY_FICC    = "ficc"
	COMPANY_CLASSIFY_RAI     = "权益"
	COMPANY_CLASSIFY_PARTNER = "合作伙伴"
)
